var _DESERIALIZER;
import { TYPE } from "./constants";
import { determineType, isSerializedType } from "./common";
import { deserializeFunction, deserializeError, deserializePromise, deserializeRegex, deserializeDate, deserializeArray, deserializeObject, deserializeString, deserializeNumber, deserializeBoolean, deserializeNull, deserializeUndefined } from "./serializers";
var DESERIALIZER = (_DESERIALIZER = {}, _DESERIALIZER[TYPE.FUNCTION] = deserializeFunction, _DESERIALIZER[TYPE.ERROR] = deserializeError, _DESERIALIZER[TYPE.PROMISE] = deserializePromise, _DESERIALIZER[TYPE.REGEX] = deserializeRegex, _DESERIALIZER[TYPE.DATE] = deserializeDate, _DESERIALIZER[TYPE.ARRAY] = deserializeArray, _DESERIALIZER[TYPE.OBJECT] = deserializeObject, _DESERIALIZER[TYPE.STRING] = deserializeString, _DESERIALIZER[TYPE.NUMBER] = deserializeNumber, _DESERIALIZER[TYPE.BOOLEAN] = deserializeBoolean, _DESERIALIZER[TYPE.NULL] = deserializeNull, _DESERIALIZER[TYPE.UNDEFINED] = deserializeUndefined, _DESERIALIZER);
var defaultDeserializers = {};
export function deserialize(str, deserializers) {
  if (deserializers === void 0) {
    deserializers = defaultDeserializers;
  }
  if (str === TYPE.UNDEFINED) {
    // the lib allows undefined returns but doesnt expect type assertions for it. need fixing
    // @ts-expect-error - we need to update the definitiona of this to allow for undefined because of this but itll take a lot of work
    return;
  }
  function replacer(key, val) {
    var _deserializers$type;
    // @ts-expect-error - function this has unknown caller
    if (isSerializedType(this)) {
      return val;
    }
    var type;
    var value;
    if (isSerializedType(val)) {
      type = val.__type__;
      value = val.__val__;
    } else {
      type = determineType(val);
      value = val;
    }
    if (!type) {
      return value;
    }

    // dynamic code execution
    // eslint-disable-next-line security/detect-object-injection
    var deserializer = (_deserializers$type = deserializers[type]) != null ? _deserializers$type : DESERIALIZER[type];
    if (!deserializer) {
      return value;
    }
    return deserializer(value, key);
  }

  // eslint-disable-next-line @typescript-eslint/no-unsafe-return
  return JSON.parse(str, replacer);
}