import type { $Values } from "utility-types";
import { TYPE } from "./constants";
import type { Thenable, CustomSerializedType, NativeSerializedType } from "./types";
import type { SerializedError, SerializedRegex, SerializedDate } from "./serializers";
type NativeSerializer<V, S, T extends $Values<typeof TYPE>> = (value: V, key: string) => NativeSerializedType<T, S>;
type CustomSerializer<V, S, T extends string> = (value: V, key: string) => CustomSerializedType<T, S>;
type PrimitiveSerializer<V, S> = (value: V, key: string) => S;
type CustomOrPrimitiveSerializer<V, T extends string> = CustomSerializer<V, unknown, T> | PrimitiveSerializer<V, unknown>;
type NativeOrCustomOrPrimitiveSerializer<V, S, T extends $Values<typeof TYPE>> = NativeSerializer<V, S, T> | CustomOrPrimitiveSerializer<V, T>;
type Serializers = {
    function?: CustomOrPrimitiveSerializer<Function, typeof TYPE.FUNCTION>;
    error?: NativeOrCustomOrPrimitiveSerializer<Error, SerializedError, typeof TYPE.ERROR>;
    promise?: CustomOrPrimitiveSerializer<Thenable, typeof TYPE.PROMISE>;
    regex?: NativeOrCustomOrPrimitiveSerializer<RegExp, SerializedRegex, typeof TYPE.REGEX>;
    date?: NativeOrCustomOrPrimitiveSerializer<Date, SerializedDate, typeof TYPE.DATE>;
    array?: CustomOrPrimitiveSerializer<readonly unknown[], typeof TYPE.ARRAY>;
    object?: CustomOrPrimitiveSerializer<Record<string, unknown>, typeof TYPE.OBJECT>;
    string?: CustomOrPrimitiveSerializer<string, typeof TYPE.STRING>;
    number?: CustomOrPrimitiveSerializer<number, typeof TYPE.NUMBER>;
    boolean?: CustomOrPrimitiveSerializer<boolean, typeof TYPE.BOOLEAN>;
    null?: CustomOrPrimitiveSerializer<undefined, typeof TYPE.NULL>;
    undefined?: CustomOrPrimitiveSerializer<undefined, typeof TYPE.UNDEFINED>;
};
export declare function serialize<T>(obj: T, serializers?: Serializers): string;
export {};
