var _SERIALIZER;
import { TYPE } from "./constants";
import { determineType, isSerializedType } from "./common";
import { serializeFunction, serializeError, serializePromise, serializeRegex, serializeDate, serializeArray, serializeObject, serializeString, serializeNumber, serializeBoolean, serializeNull, serializeUndefined } from "./serializers";
var SERIALIZER = (_SERIALIZER = {}, _SERIALIZER[TYPE.FUNCTION] = serializeFunction, _SERIALIZER[TYPE.ERROR] = serializeError, _SERIALIZER[TYPE.PROMISE] = serializePromise, _SERIALIZER[TYPE.REGEX] = serializeRegex, _SERIALIZER[TYPE.DATE] = serializeDate, _SERIALIZER[TYPE.ARRAY] = serializeArray, _SERIALIZER[TYPE.OBJECT] = serializeObject, _SERIALIZER[TYPE.STRING] = serializeString, _SERIALIZER[TYPE.NUMBER] = serializeNumber, _SERIALIZER[TYPE.BOOLEAN] = serializeBoolean, _SERIALIZER[TYPE.NULL] = serializeNull, _SERIALIZER[TYPE.UNDEFINED] = serializeUndefined, _SERIALIZER);
var defaultSerializers = {};
export function serialize(obj, serializers) {
  if (serializers === void 0) {
    serializers = defaultSerializers;
  }
  function replacer(key) {
    var _serializers$type;
    // @ts-expect-error - this has unknown caller
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, security/detect-object-injection
    var val = this[key];

    // @ts-expect-error = this has unknown caller
    if (isSerializedType(this)) {
      return val;
    }
    var type = determineType(val);
    if (!type) {
      return val;
    }

    // dynamic code execution
    // eslint-disable-next-line security/detect-object-injection
    var serializer = (_serializers$type = serializers[type]) != null ? _serializers$type : SERIALIZER[type];
    if (!serializer) {
      return val;
    }

    // @ts-expect-error val is of type any
    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
    return serializer(val, key);
  }
  var result = JSON.stringify(obj, replacer);
  if (typeof result === "undefined") {
    return TYPE.UNDEFINED;
  }
  return result;
}